
require 'claide'
require 'json'
require 'deal/utils/deal_config'
module Deal

    class Command < CLAide::Command
        include DealUtils
        self.summary = 'Export a git repository and lfs files.'
        # self.plugin_prefixes = %w(claide cocoapods)
        self.arguments = [
          # CLAide::Argument.new('TYPE', false )
        ]
        # self.abstract_command = true
        self.version = VERSION
        self.command = 'deal'
        self.description = <<-DESC
            字符/文件/目录的匹配、替换、异常功能，
            --config参数用于通过json输入配置，可以是json数组的形式表示多个配置，
            --input 用于指定所搜的目录/文件，支持bash 目录通配符
            --signs 表示匹配的字符，支持通配符
            --judge_types 审判的类型，支持文件名称，目录名称，文件字符内容，静态库匹配
            --judge_action 命中匹配字符后的动作，支持remove|replace|abort|report 类型
            --replace 如果judge_action是replace的话，则此参数生效，用于指定替换字符
            --include_paths 路径白名单
            --exclude_paths 路径黑名单

        DESC

        def self.options
            [
              ['--config', 'json or input files.'],
              ['--inputs', 'text,file or dir'],
              ['--signs', 'search items'],
              ['--judge_types', 'file|dir|text|mach-o，default is text'],
              ['--judge_action', 'which actions to match word."remove|replace|abort|report",default is abort'],
              ['--replace', 'replace word'],
              ['--include_paths', 'include search paths'],
              ['--exclude_paths', 'exclude search paths']
            ].concat(super)
        end
        def initialize(argv)
            super
            configs = argv.option('config')
            @signs = argv.option('signs',"").split('|')
            @include_paths = argv.option('include_paths',"" ).split('|')
            @exclude_paths = argv.option('exclude_paths',"" ).split('|')
            @judge_types = argv.option('judge_types',"text" ).split('|')
            @replace = argv.option('replace','')
            @judge_action = argv.option('judge_action','abort')
            @configs = []
            if configs
                if FileTest.exist? configs
                    configs = File.read(configs)
                    begin
                        configs = JSON.parse(configs)
                    rescue Exception => e
                        puts e.message
                        puts e.backtrace.inspect
                    end

                    for config in configs
                        config = DealRule.new(config)
                        @configs.push config
                    end
                else
                    logE "file:#{configs} not exist"
                    return
                end
            else
                config = {}
                config["signs"] = @signs
                config["include_paths"] = @include_paths
                config["exclude_paths"] = @exclude_paths
                config["judge_types"] = @judge_types
                config["replace"] = @replace
                config["judge_action"] = @judge_action
                config = DealRule.new(config)
                @configs.push config
            end


            @judge_count = 0
            @total_count = 0
            @progress = 0.0

            @jump_file = {}
            @results = []
            inputs = argv.option('inputs',"").split('|')
            @inputs = []
            for input in inputs
                ls = Dir.glob(input,File::FNM_DOTMATCH|File::FNM_PATHNAME)
                ls = ls.map { |item|
                    item = Pathname.new item
                    if item.basename.to_s == '.' || item.basename.to_s == '..'
                        item = item.dirname
                    end
                    item.to_s
                }
                @inputs = @inputs.concat(ls)
            end
            if @inputs.length > 1
                i = 0
                while i<@inputs.length
                    base = @inputs[i]
                    j = 0
                    while j<@inputs.length
                        if  @inputs[i]!= @inputs[j] && @inputs[j].index(@inputs[i]) == 0
                            @inputs.delete_at j
                            j-=1
                        end
                        j+=1
                    end
                    i += 1
                end
            end

        end

        def validate!
            super
            help! 'Please specify an config' unless @configs.length > 0
            help! 'Please specify an input' unless @inputs

        end

        # def handle_text(path)
        #     line_n = 0
        #     File.open(path, "r") do |aFile|
        #         aFile.each do |line|
        #             for config in @configs
        #                 if result = config.judge_file_line(line)
        #                     result.set_line_n(line_n)
        #                     @results.push result
        #                 end
        #             end
        #             line_n+=1
        #         end
        #     end
        # end


        def judeg_file_line(configs,path)
            line_configs = []
            for config in configs
                line_configs.push config if config.judge_text_line?
            end
            if line_configs.length > 0
                lines = []
                File.open(path, "r") do |aFile|
                    is_binary = false
                    while line=aFile.gets
                        need_replace = false
                        for i in 0...(line.length)
                            begin
                                if line[i].ord == 0
                                    is_binary = true
                                    break
                                end
                            rescue Exception =>e
                                is_binary = true
                                # logE "file prase error:#{e.to_s},path:#{path}"
                                break
                            end

                        end
                        if is_binary
                            break
                        end

                        for config in line_configs
                            replace = config.judge_line path,line,lines.length
                            if replace && line != replace
                                need_replace
                            end
                        end

                        if !need_replace
                            lines.push line
                        end
                    end
                end
            end

        end
        def deal_item(configs,input)
            @judge_count += 1
            progress = @judge_count*1.0/@total_count
            if progress - @progress > 0.01
                @progress = progress
                logW "处理进度:#{format("%.2f",progress*100).to_f}%"
            end

            configs = []+configs
            line_configs = []
            multi_configs = []
            directory = File.directory?(input)
            for config in configs
                if directory
                    if !config.judge_dir_path(input)
                        configs.delete config
                    end
                else
                    config.judge_file_path(input)
                end
                if config.judge_text_line?
                    line_configs.push config
                end
                if config.judge_text_multi?
                    multi_configs.push config
                end
            end


            if !directory
                if line_configs.length > 0
                    judeg_file_line(line_configs,input)
                end
            else
                Dir.entries(input).each do |sub|
                    if sub != '.' && sub != '..'
                        deal_item(configs,"#{input}/#{sub}")
                    else
                        @judge_count += 0.5
                    end
                end
            end
        end

        def run
            logN "--inputs glob:\n "+@inputs.join("\n")
            logN '--configs'
            for config in @configs
                logN config.to_s verbose?
            end

            for input in @inputs
                if FileTest.directory? input
                    match = Pathname.new(input).join('**/*').to_s
                    @total_count += Dir.glob(match,File::FNM_DOTMATCH).size
                else
                    @total_count += 1
                end
            end
            logW "开始扫描，总共#{@total_count}个文件"
            for input in @inputs
                if FileTest.exist?(input)
                    deal_item @configs, input
                else
                    logE input+' file does not exist'
                end
            end

            progress = @judge_count*1.0/@total_count
            logW "处理进度:100%"

            logN '=============================='
            logN "共进行了#{@judge_count}次匹配"
            for config in @configs
                logW  "config:#{config.to_s verbose?}"
                if config.results.length == 0
                    logN '没有匹配到结果'
                else
                    logN "匹配到 #{config.results.length} 个结果，如下："
                    for result in config.results
                        logN result.to_s verbose?
                    end
                end
                logN '=============================='
            end

        end
    end
end
